# -*- coding: utf-8 -*-
# Copyright (c) 2003 The Connexions Project, All Rights Reserved
# Written by Brent Hendricks

from Products.CMFDiffTool.interfaces import IDifference  # NOQA
from Products.CMFDiffTool.interfaces import IDiffTool as portal_diff  # NOQA
